@@ -1,41 +1,6 @@
 /*
- * Copyright (c) 2005, 2006, 2007
- *	Simson L. Garfinkel and Basis Technology, Inc. 
- *      All rights reserved.
- *
- * This code is derrived from software contributed by
- * Simson L. Garfinkel
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Simson L. Garfinkel
- *    and Basis Technology Corp.
- * 4. Neither the name of Simson Garfinkel, Basis Technology, or other
- *    contributors to this program may be used to endorse or promote
- *    products derived from this software without specific prior written
- *    permission.
- *
- * THIS SOFTWARE IS PROVIDED BY SIMSON GARFINKEL, BASIS TECHNOLOGY,
- * AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL SIMSON GARFINKEL, BAIS TECHNOLOGy,
- * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.  
+ * afflib.cpp
+ * Distributed under the Berkeley 4-part license
  */
 
 #include "affconfig.h"
@@ -75,15 +40,15 @@ struct af_vnode *af_vnode_array[] = {
 #ifdef USE_S3
     &vnode_s3,				// must be first for s3:// interpertation
 #endif
-    &vnode_afd, 
+    &vnode_afd,
     &vnode_afm,				// must be before aff
     &vnode_aff,
 #ifdef USE_QEMU
     &vnode_vmdk,
     &vnode_dmg,
 #endif
-#ifdef USE_SPARSEIMAGE			
-    &vnode_sparseimage,           
+#ifdef USE_SPARSEIMAGE
+    &vnode_sparseimage,
 #endif
     &vnode_split_raw,			// must be before raw
     &vnode_raw,				// greedy; must be last
@@ -169,7 +134,7 @@ void	af_perror(const char *str)
     fprintf(stderr,"%s: %s: %s\n",getprogname(),str,af_error_str);
 #else
     fprintf(stderr,"%s: %s\n",str,af_error_str);
-#endif    
+#endif
 }
 
 
@@ -264,7 +229,7 @@ AFFILE *af_open_with(const char *url,int flags,int mode, struct af_vnode *v)
     AFFILE *af = (AFFILE *)calloc(sizeof(AFFILE),1);
     af_crypto_allocate(af);
 #ifdef HAVE_PTHREAD
-    pthread_rwlock_init(&af->rwlock);
+    pthread_rwlock_init(&af->rwlock, 0);
     AF_WRLOCK(af);
 #endif
     af->v	  = v;
@@ -273,7 +238,6 @@ AFFILE *af_open_with(const char *url,int flags,int mode, struct af_vnode *v)
     af->openmode  = mode;
     af->image_sectorsize = 512;		// default size
     af->error_reporter = warnx;
-    af->badflag   = (unsigned char *)malloc(af->image_sectorsize);
 
     /* Decode URL */
     af_parse_url(url,&af->protocol,&af->hostname,&af->username,&af->password,
@@ -319,7 +283,7 @@ AFFILE *af_open_with(const char *url,int flags,int mode, struct af_vnode *v)
 	    af->password[buflen] = '\000';
 	}
     }
-		
+
     /* TK: If no password was set and the AFFLIB_ASK_PASS is set, ask for a passphrase */
 
     /* Note things for hard files */
@@ -339,7 +303,7 @@ AFFILE *af_open_with(const char *url,int flags,int mode, struct af_vnode *v)
     if(flags & AF_HALF_OPEN) return af;	// for low-level tools
 
     /* Try opening it! */
-    if((*af->v->open)(af)){ 
+    if((*af->v->open)(af)){
 	strlcpy(af_error_str,af->error_str,sizeof(af_error_str)); // make a copy of the error string
 	af_deallocate(af);
 	return 0;
@@ -372,7 +336,7 @@ AFFILE *af_open_with(const char *url,int flags,int mode, struct af_vnode *v)
 		af_sanitize_password(af);
 	    }
 	}
-	
+
 	/* Try public key... */
 	if(can_decrypt==false){
 	    const char *kf = getenv(AFFLIB_DECRYPTING_PRIVATE_KEYFILE);
@@ -381,7 +345,7 @@ AFFILE *af_open_with(const char *url,int flags,int mode, struct af_vnode *v)
 	    }
 	}
     }
-	
+
     af_read_sizes(af);		// set up the metadata
     if(af_trace) fprintf(af_trace,"af_open_with(%s,%o,%o,%s)\n",url,flags,mode,v->name);
     return af;
@@ -552,10 +516,7 @@ int af_eof(AFFILE *af)
 
     if(af_vstat(af,&vni)) return -1;	// this is bad; we need vstat...
     if(vni.use_eof) return vni.at_eof;	// if implementation wants to use it, use it
-    if(af->pos<0){			// pos shouldn't be <0
-	errno = EINVAL;
-	return -1;		// this is bad
-    }
+
     int ret = (int64_t)af->pos >= (int64_t)vni.imagesize;
     AF_UNLOCK(af);
     return ret;
@@ -572,7 +533,7 @@ void af_set_callback(AFFILE *af,void (*wcb)(struct affcallback_info *))
 void af_enable_compression(AFFILE *af,int type,int level)
 {
     AF_WRLOCK(af);
-    af->compression_type  = type; 
+    af->compression_type  = type;
     af->compression_level = level;
     AF_UNLOCK(af);
 }
@@ -623,7 +584,7 @@ int64_t af_get_imagesize(AFFILE *af)
     return ret;
 }
 
-/* 
+/*
  * af_make_badflag:
  * Create a randomized bag flag and
  * leave an empty segment of how many badsectors there are
@@ -631,13 +592,16 @@ int64_t af_get_imagesize(AFFILE *af)
  */
 int af_make_badflag(AFFILE *af)
 {
+    if(af->badflag!=0) free(af->badflag);
+    af->badflag = (unsigned char *)malloc(af->image_sectorsize); // current sector size
+
 #ifdef HAVE_RAND_pseudo_bytes
     /* Use a good random number generator if we have it */
     RAND_pseudo_bytes(af->badflag,af->image_sectorsize);
     strcpy((char *)af->badflag,"BAD SECTOR");
 #else
     /* Otherwise use a bad one */
-    for(int i=0;i<af->image_sectorsize;i++){
+    for(uint32_t i=0;i<af->image_sectorsize;i++){
       af->badflag[i] = rand() & 0xff;
     }
 #endif
@@ -688,16 +652,16 @@ void af_aes_decrypt(AFFILE *af,const char *segname,unsigned char *data,size_t *d
     if(datalen==0) return;		// can't decrypt; no clue how long it is
 
     /* An encrypted segment was retrieved; decrypt and trunc the length as necessary */
-    unsigned int extra = (*datalen) % AES_BLOCK_SIZE;
-    unsigned int pad = (AES_BLOCK_SIZE - extra) % AES_BLOCK_SIZE;
-    
+    uint32_t extra = (*datalen) % AES_BLOCK_SIZE;
+    uint32_t pad = (AES_BLOCK_SIZE - extra) % AES_BLOCK_SIZE;
+
     if(data==0){			// just wants to find out new length
 	if(extra>0){
 	    *datalen -= AES_BLOCK_SIZE;
 	}
 	return;
     }
-	
+
     if(extra!=0 && *datalen < AES_BLOCK_SIZE){
 	*datalen = 0;			// something is wrong
 	return;
@@ -713,7 +677,7 @@ void af_aes_decrypt(AFFILE *af,const char *segname,unsigned char *data,size_t *d
     unsigned char iv[AES_BLOCK_SIZE];
     memset(iv,0,sizeof(iv));
     strlcpy((char *)iv,segname,sizeof(iv));
-	
+
     /* Decrypt! */
     AF_READLOCK(af);
     AES_cbc_encrypt(data,data,*datalen,&af->crypto->dkey,iv,AES_DECRYPT);
@@ -723,7 +687,7 @@ void af_aes_decrypt(AFFILE *af,const char *segname,unsigned char *data,size_t *d
 }
 
 
-int af_get_seg(AFFILE *af,const char *segname,unsigned long *arg,unsigned char *data,size_t *datalen)
+int af_get_seg(AFFILE *af,const char *segname,uint32_t *arg,unsigned char *data,size_t *datalen)
 {
     AF_READLOCK(af);
     if(af->v->get_seg==0){
@@ -763,14 +727,14 @@ int af_get_seg(AFFILE *af,const char *segname,unsigned long *arg,unsigned char *
 	    return 0;			// finally it fits
 	}
     }
-#endif    
+#endif
     /* Try for the unencrypted segment */
     int ret = (*af->v->get_seg)(af,segname,arg,data,datalen);
     AF_UNLOCK(af);
     return ret;
 }
 
-int af_get_next_seg(AFFILE *af,char *segname,size_t segname_len,unsigned long *arg,
+int af_get_next_seg(AFFILE *af,char *segname,size_t segname_len,uint32_t *arg,
 			unsigned char *data,size_t *datalen)
 {
     size_t datalen_orig = datalen ? *datalen : 0;
@@ -782,7 +746,7 @@ int af_get_next_seg(AFFILE *af,char *segname,size_t segname_len,unsigned long *a
     }
     int r = (*af->v->get_next_seg)(af,segname,segname_len,arg,data,datalen);
 #ifdef HAVE_AES_ENCRYPT
-    if(AF_SEALING_VNODE(af) 
+    if(AF_SEALING_VNODE(af)
        && ends_with(segname,AF_AES256_SUFFIX)
        && af->crypto->auto_decrypt){
 	segname[strlen(segname)-strlen(AF_AES256_SUFFIX)] = 0;
@@ -806,7 +770,7 @@ int af_get_next_seg(AFFILE *af,char *segname,size_t segname_len,unsigned long *a
 	AF_UNLOCK(af);
 	return r;			// not sure why we got this error
     }
-#endif    
+#endif
     AF_UNLOCK(af);
     return r;
 }
@@ -829,9 +793,10 @@ int af_rewind_seg(AFFILE *af)
  */
 
 int af_update_segf(AFFILE *af, const char *segname,
-		  unsigned long arg,const u_char *data,unsigned int datalen,u_int flag)
+		  uint32_t arg,const u_char *data,uint32_t datalen,uint32_t flag)
 {
-    if(af_trace) fprintf(af_trace,"af_update_segf(%p,segname=%s,arg=%lu,datalen=%d)\n",af,segname,arg,datalen);
+    if(af_trace) fprintf(af_trace,"af_update_segf(%p,segname=%s,arg=%"PRIu32",datalen=%d)\n",
+			 af,segname,arg,datalen);
     AF_WRLOCK(af);
     if(af->v->update_seg==0){
 	errno = ENOTSUP;
@@ -861,10 +826,10 @@ int af_update_segf(AFFILE *af, const char *segname,
 	oldname = segname;
 	segname = aesname;
 
-	/* Figure out if we need to padd out for encryption. Allocate space and 
+	/* Figure out if we need to padd out for encryption. Allocate space and
 	 */
-	unsigned int extra = (datalen) % AES_BLOCK_SIZE;
-	unsigned int pad = (AES_BLOCK_SIZE - extra) % AES_BLOCK_SIZE;
+	uint32_t extra = (datalen) % AES_BLOCK_SIZE;
+	uint32_t pad = (AES_BLOCK_SIZE - extra) % AES_BLOCK_SIZE;
 	newdata = (unsigned char *)malloc(datalen+pad+extra);
 	memset(newdata+datalen,pad+extra,pad); // PKCS7 uses 01 for one pad byte, 02 02 for two, etc.
 	/* Encrypt */
@@ -876,10 +841,13 @@ int af_update_segf(AFFILE *af, const char *segname,
     }
 #endif
     int r = (*af->v->update_seg)(af,segname,arg,data,datalen); // actually update the segment
-    if(r==0) af->bytes_written += datalen;
+    if(r < 0)
+        { AF_UNLOCK(af); return r; }
+
+    af->bytes_written += datalen;
 #ifdef HAVE_AES_ENCRYPT
     /* if we encrypted, make sure the unencrypted segment is deleted */
-    if(r==0 && oldname) (*af->v->del_seg)(af,oldname);
+    if(oldname) (*af->v->del_seg)(af,oldname);
     if(newdata){
 	free(newdata);		// free any allocated data
 	newdata = 0;
@@ -902,7 +870,6 @@ int af_update_segf(AFFILE *af, const char *segname,
 #ifdef USE_AFFSIGS
     const u_char *signdata = data;	// remember the original data location
     if(AF_SEALING(af)
-       && (r==0)
        && af->crypto->sign_privkey
        && ((flag & AF_SIGFLAG_NOSIG)==0)
        && !ends_with(segname,AF_SIG256_SUFFIX)){
@@ -916,14 +883,14 @@ int af_update_segf(AFFILE *af, const char *segname,
 
 /* Requires no locking because locking is done in af_update_segf */
 int af_update_seg(AFFILE *af, const char *segname,
-		  unsigned long arg,const u_char *data,unsigned int datalen)
+		  uint32_t arg,const u_char *data,uint32_t datalen)
 {
     return af_update_segf(af,segname,arg,data,datalen,0);
 }
 
 #ifdef HAVE_OPENSSL_BIO_H
 /* Requires no locking */
-int	af_update_seg_frombio(AFFILE *af,const char *segname,unsigned long arg,BIO *bio)
+int	af_update_seg_frombio(AFFILE *af,const char *segname,uint32_t /*arg*/,BIO *bio)
 {
     /* Get the buffer to write out */
     u_char *buf=0;
